/**
 *    @file  base64.h
 *   @brief Simple Base64 encoding and decoding functions
 *
 *  Copied and adapted from https://github.com/elzoughby/Base64
 *
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  25/10/2024
 *      Revision:  none
 * Last modified:  2024-11-04 12:16
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifndef __BASE64_H__
#define __BASE64_H__

/**
 *  @brief  Encode a zero terminated C string in Base64
 *
 *  @param  plain Zero Terminated C string
 *  @return Encoded zero terminated C string
 *      @retval Pointer to a zero terminated C string
 *      @retval NULL in case of out of memory
 *
 *  plain can not be NULL. the returned value is mallocated and needs to be
 *  freed.
 */
char* base64_encode(char* plain);

/**
 *  @brief  Decode a zero terminated C Base64 encoded string
 *
 *  @param  cipher Zero Terminated C Base64 encoded string
 *  @return Decoded zero terminated C string
 *      @retval Pointer to a zero terminated C string
 *      @retval NULL in case of out of memory
 *
 *  cipher can not be NULL. the returned value is mallocated and needs to be
 *  freed.
 */
char* base64_decode(char* cipher);

#endif /* __BASE64_H__ */

/* vim: set tw=80: */
