/**
 *    @file  clusterlst.c
 *   @brief  Basic(non-thread-safe) single-chained list of record with sentinal
 *
 *  <+DETAILED+>
 *
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  25/10/2024
 *      Revision:  none
 * Last modified:  2024-11-10 09:38
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "clusterlst.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

typedef struct clusterrecord_s {
    struct clusterrecord_s* next;
    cluster_t* cluster;
} clusterrecord_t;

static clusterrecord_t* clusterlistfirst;
static clusterrecord_t* clusterlistlast;
static clusterrecord_t* clusterlistcursor;

static void clusterlist_add_postinit (cluster_t* cluster);
static cluster_t* clusterlist_find_postinit (const char* host);
static cluster_t* clusterlist_first_postinit();
static cluster_t* clusterlist_next_postinit();
static cluster_t* clusterlist_get_postinit();

static void clusterlist_init() {
    if (NULL==(clusterlistfirst=malloc(sizeof(struct clusterrecord_s)))) {
        perror("clusterlist_init");
        exit(EXIT_FAILURE);
    }
    /* Initialize chained structure */
    clusterlistlast = clusterlistfirst;
    clusterlistcursor = clusterlistfirst;
    clusterlistlast->next=NULL;
    clusterlistlast->cluster=NULL;

    /* From now, the application can use the real functions */
    clusterlist_add = clusterlist_add_postinit;
    clusterlist_find = clusterlist_find_postinit;
    clusterlist_first = clusterlist_first_postinit;
    clusterlist_next = clusterlist_next_postinit;
    clusterlist_get = clusterlist_get_postinit;
}

static void clusterlist_add_postinit (cluster_t* cluster) {
    /* Store the record in the sentinel */
    clusterlistlast->cluster=cluster;

    /* Create a new sentinel */
    if (NULL==(clusterlistlast->next=malloc(sizeof(struct clusterrecord_s)))) {
        perror("clusterlist_add_postinit malloc(sentinel)");
        exit(EXIT_FAILURE);
    }
    clusterlistlast = clusterlistlast->next;
    clusterlistlast->next=NULL;
    clusterlistlast->cluster=NULL;
}
static void clusterlist_add_preinit (cluster_t* cluster) {
    clusterlist_init();
    clusterlist_add(cluster);
}
void (*clusterlist_add)(cluster_t* cluster) = clusterlist_add_preinit;

static cluster_t* clusterlist_find_postinit(const char* host) {
    clusterlistcursor = clusterlistfirst;
    while (clusterlistcursor->cluster&&
            (strcmp(host,clusterlistcursor->cluster->host)))
        clusterlistcursor = clusterlistcursor->next;
    return (clusterlistcursor->cluster);
}
static cluster_t* clusterlist_find_preinit(const char* host) {
    clusterlist_init();
    return clusterlist_find(host);
}
cluster_t* (*clusterlist_find)(const char* host) = clusterlist_find_preinit;

static cluster_t* clusterlist_first_postinit() {
    clusterlistcursor = clusterlistfirst;
    return (clusterlistcursor->cluster);
}
static cluster_t* clusterlist_first_preinit() {
    clusterlist_init();
    return clusterlist_first();
}
cluster_t* (*clusterlist_first)() = clusterlist_first_preinit;

static cluster_t* clusterlist_next_postinit() {
    if (clusterlistcursor->next)
        clusterlistcursor = clusterlistcursor->next;
    return (clusterlistcursor->cluster);
}
static cluster_t* clusterlist_next_preinit() {
    clusterlist_init();
    return clusterlist_next();
}
cluster_t* (*clusterlist_next)() = clusterlist_next_preinit;

static cluster_t* clusterlist_get_postinit() {
    return (clusterlistcursor->cluster);
}
static cluster_t* clusterlist_get_preinit() {
    clusterlist_init();
    return clusterlist_get();
}
cluster_t* (*clusterlist_get)() = clusterlist_get_preinit;

/* vim: set tw=80: */
