/**
 *    @file  json.c
 *   @brief  Wrapper around cJSON library with helpers
 *
 *  This library is only a simple wrapper around the cJSON library, providing
 *  helper functions.
 *
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  29/10/2024
 *      Revision:  none
 * Last modified:  2024-11-05 19:02
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "json.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char* json2text(cJSON* value_json) {
    char* retval;
    if (cJSON_IsNull(value_json))
        retval = strdup("null");
    else if (cJSON_IsString(value_json))
        retval = strdup(value_json->valuestring);
    else if (cJSON_IsNumber(value_json)) {
        retval = (char*)malloc(20);
        if (retval)
            snprintf(retval,20,"%d",value_json->valueint);
    } else if (cJSON_IsBool(value_json)) {
        if (cJSON_IsTrue(value_json))
            retval = strdup("true");
        else
            retval = strdup("false");
    } else if (cJSON_IsArray(value_json)) {
        retval = cJSON_PrintUnformatted(value_json);
    } else if (cJSON_IsObject(value_json)) {
        retval = cJSON_PrintUnformatted(value_json);
    } else
        retval = strdup("Invalid data");
    return retval;
}

/* vim: set tw=80: */
